from __future__ import annotations


class AgentError(Exception):
    """Base class for exceptions in this module."""

    message: str

    def __init__(self, message: str):
        self.message = message
        super().__init__(self.message)


class RoomNotCreatedError(AgentError):
    """Exception raised for errors in the Agent Room, when Room is not created."""

    def __init__(
        self,
    ):
        super().__init__(
            """
            Room Creation Failed, Possible ways to fix this:
            - Call the `agen.join()` before calling `agent.connect()` to the Room
            - Make sure the API_KEY and Project_ID are valid before creating Room.
            - Make Sure the room_id is valid for the API_KEY being used to connect to the Room.
            """
        )


class RoomNotConnectedError(AgentError):
    """Exception raised for errors in the Agent Room, when Room is not connected."""

    def __init__(
        self,
    ):
        super().__init__(
            """
            Room is not created, Possible ways to fix this:
            - Call the `agent.join()` before calling `agent.connect()` to the Room
            - Make sure the API_KEY and Project_ID are valid before creating Room.
            - Make Sure the room_id is valid for the API_KEY being used to connect to the Room.
            """
        )



from pydantic import BaseModel, Field
from rtc import RTC, RTCOptions
from ._exceptions import RoomNotConnectedError, RoomNotCreatedError


class AgentOptions(BaseModel):
    rtc_options: RTCOptions = Field(
        description="RTC Options for the Agent",
    )


class Agent:
    """
    Agents is defined as the higher level user which is its own entity and has exposed APIs to
    interact with different Models and Outer World using dRTC.

    Agents can be connected to the dRTC Network by two stepsL
     - Joining the Agent to the dRTC Network, which assigns the Agent to a Room, using this Agent can setup event Listeners before connecting to the Room.
     - Connecting the Agent to the Room, which allows the Agent to send and receive data from the Room, and become part of the Room.
    """

    def __init__(self, options: AgentOptions):
        # Options is the configuration for the Agent.
        self.options = options

        # RTC is the Real Time Communication Handler for the Agent.
        self.__rtc = RTC(options.rtc_options)

    @property
    def rtc(self):
        return self.__rtc

    @property
    def room(self):
        """
        Room is the Room the Agent is connected to, This is only available after the Agent is connected.

        raise ValueError if Agent is not connected to the Room.
        """
        if not self.__rtc.room:
            raise RoomNotCreatedError()

        return self.__rtc.room

    async def join(self):
        """
        Joins the Agent to the dRTC Network, Upon Joining the Agent is assigned to a Room.
        Using the Room the Agent can Listen to Events such as RemotePeer Joining, Leaving and Produce Events Triggered in the Room
        """
        room = await self.__rtc.create()

        if not room:
            raise RoomNotConnectedError()

        return room

    async def connect(self):
        """
        Connects the Agent to the Room, This is only available after the Agent is joined to the dRTC Network.
        """
        room = self.__rtc.room

        if not room:
            raise RoomNotCreatedError()

        await room.connect()



import json
import logging
from pydantic import BaseModel, Field
from huddle01 import (
    HuddleClient,
    HuddleClientOptions,
    Role,
    AccessTokenOptions,
    AccessToken,
    AccessTokenData,
)

logging.basicConfig(level=logging.INFO)

logger = logging.getLogger("RTC")


class RTCOptions(BaseModel):
    """
    RTCOptions is the configuration for the Room.
    """

    project_id: str = Field(
        description="Huddle Project Id",
    )

    api_key: str = Field(
        description="Huddle API Key",
    )

    room_id: str = Field(
        description="Huddle Room Id to Join",
    )

    huddle_client_options: HuddleClientOptions = Field(
        description="Huddle Client Options",
    )

    metadata: dict[str, str] = Field(
        default_factory=dict,
        description="Metadata to be sent to the Room",
    )

    role: str = Field(
        description="Role of the User",
    )


class RTC:
    """
    RTC Handles anything related to Real Time Communication, using the dRTC Network,
    Currently it is using Huddle01 as the dRTC Network.
    """

    def __init__(self, options: RTCOptions):
        logger.info("Initializing RTC Room: ", options.room_id)

        self._logger = logger.getChild(options.room_id)

        self._options = options

        self._huddle_client = HuddleClient(
            project_id=self._options.project_id,
            options=self._options.huddle_client_options,
        )

    def __str__(self):
        return f"RTC Room: {self._options.room_id}"

    def __repr__(self):
        return f"RTC Room: {self._options.room_id}"

    @property
    def options(self):
        return self._options

    @property
    def huddle_client(self):
        return self._huddle_client

    @property
    def room(self):
        return self._huddle_client.room

    async def create(self):
        """
        Join Room is used to join the Room with the given Room Id,

        Exceptions:
            - If there is any error while joining the Room, it will raise an Exception.
        """
        self._logger.info("Creating Huddle01 Room ", self._options.room_id)

        accessTokenData = AccessTokenData(
            room_id=self._options.room_id,
            api_key=self._options.api_key,
            options=AccessTokenOptions(
                metadata=json.dumps(self._options.metadata),
            ),
            role=Role(self._options.role),
        )

        accessToken = AccessToken(
            data=accessTokenData,
        )

        try:
            token = await accessToken.to_jwt()

            room = await self.huddle_client.create(self._options.room_id, token)

            return room
        except Exception as e:
            logger.error("Error while joining the Room: ", e)
            raise e
